--luacheck: globals log
local hasbobplates
if _G.bobmods and _G.bobmods.plates then hasbobplates=true else hasbobplates=false end
if hasbobplates then log("OmnibarrelsExpanded found BobPlates") end

--I may have localed all the things :)

--requires 3x entries to locale:
--[[
[item-name]
filled-bottle=__1__ bottle

[recipe-name]
empty-filled-bottle=Empty __1__ bottle
fill-bottle=fill __1__ bottle
--]]

local barrel_size = 25 -- default 25
local barrel_stack = 10 -- default 10
local barrel_fill_energy = 1
local barrel_empty_energy = 1

-- if there are more fluids than this number, all barrel filling recipes move to new tab
-- base game has 7 fluids
local new_tab_threshold = 7

-- use alternative, bright colour overlay?
local high_contrast = false

-- opacity of colour overlay
local body_alpha = 0.75
local hoop_alpha = 0.75

--define variables here in case we don't have bobplates.
local locfill, locempty, locfilled = "fill-barrel", "empty-filled-barrel", "filled-barrel"
local item_icon = "__OmnibarrelsExpanded__/graphics/icon/empty-barrel.png"
local item_fill_icon = "__OmnibarrelsExpanded__/graphics/icon/fill-barrel-recipe.png"
local item_empty_icon = "__OmnibarrelsExpanded__/graphics/icon/empty-barrel-recipe.png"
local usecategory = "crafting-with-fluid"
local techname = "fluid-handling"
local barrel_subgroup = "barrel"
local fill_subgroup = "barrel"
local empty_subgroup = "barrel"
local suffix = "-barrel"
local barrel_type="empty-barrel"

local body_mask, hoop_mask
if high_contrast then
  body_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-body-mask-white.png"
  hoop_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-hoop-mask-white.png"
else
  body_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-body-mask.png"
  hoop_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-hoop-mask.png"
end

-- remove crude oil barrels from fluid handling, going to be re-adding them below
-- not strictly necessary, but tidy
for i,e in pairs(data.raw["technology"]["fluid-handling"].effects) do
  if e["recipe"] == "fill-crude-oil-barrel" or
    e["recipe"] == "empty-crude-oil-barrel" then
    data.raw["technology"]["fluid-handling"].effects[i] = nil
  end
end

-- get total number of fluids
local fluid_count = 0
for _, _ in pairs(data.raw.fluid) do
  fluid_count = fluid_count + 1
end

if not hasbobplates then
  if fluid_count > new_tab_threshold then
    data:extend({
      {
        type = "item-subgroup",
        name = "fill-barrel",
        group = "fluids",
        order = "b"
      },
      {
        type = "item-subgroup",
        name = "empty-barrel",
        group = "fluids",
        order = "c"
      }
    })
    fill_subgroup = "fill-barrel"
    empty_subgroup = "empty-barrel"
  end
elseif hasbobplates then
    --Remove bobs oil fill and empty as OmnibarrelsExpanded autogenerates it.
    --bobplates requires bob library so we don't have to explicitly check for it here
    _G.bobmods.lib.tech.remove_recipe_unlock("barrels", "bob-fill-crude-oil-barrel")
    _G.bobmods.lib.tech.remove_recipe_unlock("barrels", "bob-empty-crude-oil-barrel")
end
if _G.angelsmods and _G.angelsmods.petrochem then
  --remove all current angels barrel tech unlocks to replace later!
  data.raw.technology["gas-oil-barreling"].effects={}
  local machine = data.raw["assembling-machine"]["heavy-pump"]
  _G.bobmods.lib.machine.add_category(machine, "air-pump")
  _G.bobmods.lib.machine.add_category(machine, "water-pump")
end

--use tidied-up icon for empty barrels
--moved below bobplates check for stack_size
data.raw.item["empty-barrel"].icon = "__OmnibarrelsExpanded__/graphics/icon/empty-barrel.png"
data.raw.item["empty-barrel"].stack_size = barrel_stack

-------------------------------------------------------------------------------
-- adapted from factorio stdlib
local function string_contains(s, ends)
    return s and string.find(s, ends) ~= nil
end

-------------------------------------------------------------------------------
-- generate filled barrel item and recipes for each fluid
for _, f in pairs(data.raw.fluid) do
  if hasbobplates and not (f.name == "liquid-fuel" or f.name == "ferric-chloride-solution") then

  --log("Fluid found " .. f.name)

  --Do these barrels exist?
  --currently only checking for bob's canisters.
  --local existing_barrel=data.raw.item[f.name.."-barrel"]
  local existing_canister=data.raw.item[f.name.."-canister"]


  --Lets do a crude check for Gas/air versus liquid if bobplates is present
  --flip variables around depending on presence
  --local possiblygas = false
  if hasbobplates then
    barrel_subgroup = "bob-barrel"
    fill_subgroup  ="bob-barrel"
    empty_subgroup = "bob-empty-barrel"
    usecategory = "water-pump"
    techname = "barrels"
    suffix = "-barrel"
    barrel_type = "empty-barrel"
    item_icon = "__OmnibarrelsExpanded__/graphics/icon/empty-barrel.png"
    item_fill_icon = "__OmnibarrelsExpanded__/graphics/icon/fill-barrel-recipe.png"
    item_empty_icon = "__OmnibarrelsExpanded__/graphics/icon/empty-barrel-recipe.png"
    barrel_stack = 20
    barrel_size = 25
    locfill, locempty, locfilled = "fill-barrel", "empty-filled-barrel", "filled-barrel"
    if high_contrast then
      body_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-body-mask-white.png"
      hoop_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-hoop-mask-white.png"
    else
      body_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-body-mask.png"
      hoop_mask = "__OmnibarrelsExpanded__/graphics/icon/barrel-hoop-mask.png"
    end
      --Lets assume that if gas, air, or steam is contained in the name it is a gas product
      if string_contains(f.name, "gas-") or string_contains(f.name, "-air") or string_contains(f.name, "steam") or string_contains(f.name, "-gas") or existing_canister then
        barrel_subgroup = "bob-gas-bottle"
        fill_subgroup  = "bob-gas-bottle"
        empty_subgroup = "bob-empty-gas-bottle"
        usecategory = "air-pump"
        techname = "gas-canisters"
        suffix = "-canister"
        barrel_type = "gas-canister"

       --change these if item masks are created for canisters
        --item_icon = "__bobplates__/graphics/icons/gas-canister.png"
        item_icon = "__OmnibarrelsExpanded__/graphics/icon/gas-canister.png"
        item_fill_icon = "__OmnibarrelsExpanded__/graphics/icon/fill-canister.png"
        item_empty_icon = "__OmnibarrelsExpanded__/graphics/icon/empty-canister.png"
          if high_contrast then
            body_mask = "__OmnibarrelsExpanded__/graphics/icon/canister-body-mask.png"
            hoop_mask = "__OmnibarrelsExpanded__/graphics/icon/canister-hoop-mask-white.png"
          else
            body_mask = "__OmnibarrelsExpanded__/graphics/icon/canister-body-mask.png"
            hoop_mask = "__OmnibarrelsExpanded__/graphics/icon/canister-hoop-mask.png"
          end

        --possiblygas=true
        barrel_stack=100
        barrel_size=5
        locfill, locempty, locfilled = "fill-bottle", "empty-filled-bottle", "filled-bottle"
        if existing_canister then
          _G.bobmods.lib.tech.remove_recipe_unlock("gas-canisters", f.name .. suffix)
        end
        if data.raw["item"][f.name .. "-barrel"] then
          --log("REMOVING barrel for " .. f.name)
          data.raw["item"][f.name.."-barrel"].hidden=true
        end
        if data.raw["recipe"]["fill-".. f.name .. "-barrel"] then
          --log("REMOVING fill barrel for " .. f.name)
          data.raw["recipe"]["fill-".. f.name.."-barrel"].hidden=true
        end
        if data.raw["recipe"]["empty-".. f.name .. "-barrel"] then
          --log("REMOVING empty barrel for " .. f.name)
          data.raw["recipe"]["empty-".. f.name.."-barrel"].hidden=true
        end
        log("setting " ..f.name .." as Gas")
      else
        log("Setting ".. f.name .." as Liquid")
      end
  else
    log("Setting " .. f.name .. " as Liquid")
  end


  fluid_count = fluid_count + 1
  -- barrel colours from fluid colour
  local body_tint = f.base_color
  body_tint["a"] = body_alpha
  local hoop_tint = f.flow_color
  hoop_tint["a"] = hoop_alpha

  local item_name=f.name .. suffix

  -- generate item, fill recipe and empty recipe for each fluid
  --if not existing_canister then --ignore bobs gas canister items and recipes at least until a time when gotlag makes an awesome bottle icon
    --if not existing_barrel then -- don't overwrite existing barrels
    data:extend({
      {
        type = "item",
        name = f.name..suffix,
        localised_name = {"item-name."..locfilled, {"fluid-name." .. f.name}},
        icons =
        {
          {
            icon = item_icon
          },
          {
            icon = body_mask,
            tint = body_tint
          },
          {
            icon = hoop_mask,
            tint = hoop_tint
          }
        },
        flags = {"goes-to-main-inventory"},
        subgroup = barrel_subgroup,
        order = "b["..f.name..suffix.."]",
        stack_size = barrel_stack
      }
    })
    -- elseif existing_barrel then --update groups for existing barrels
    --   --if string_ends_with(f.name, "-barrel") then item_name=f.name end
    --   item_name = existing_barrel.name
    --   existing_barrel.subgroup= barrel_subgroup
    --   existing_barrel.order = "b["..f.name..suffix.."]"
    --   existing_barrel.stacksize = barrel_stack
    -- end

    --Go ahead and change all the recipes to match though!
    --kind of dirty but the easiest way since not all barrel items might have fill or empty recipes following the fill-name-barrel format. or might not have recipes at all.
      --FILL recipe
      local fill_name = "fill-"..f.name..suffix
      data:extend({
        {
          type = "recipe",
          name = fill_name,
          localised_name = {"recipe-name."..locfill, {"fluid-name." .. f.name}},
          category = usecategory,
          energy_required = barrel_fill_energy,
          subgroup = fill_subgroup,
          order = "b[fill-"..f.name..suffix.."]",
          enabled = false,
          icons =
          {
            {icon = f.icon},
            {icon = item_fill_icon}
          },
          ingredients =
          {
            {type="fluid", name=f.name, amount=barrel_size},
            {type="item", name=barrel_type, amount=1},
          },
          results=
          {
            {type="item", name=item_name, amount=1}
          }
        }
      })



      --EMPTY Recipe
      local empty_name = "empty-"..f.name..suffix
      data:extend({
        {
          type = "recipe",
          name = empty_name,
          localised_name = {"recipe-name."..locempty, {"fluid-name." .. f.name}},
          category = usecategory,
          energy_required = barrel_empty_energy,
          subgroup = empty_subgroup,
          order = "c[empty-"..f.name..suffix.."]",
          enabled = false,
          icons =
          {
            {icon = f.icon},
            {icon = item_empty_icon}
          },
          ingredients =
          {
            {type="item", name= item_name, amount=1}
          },
          results=
          {
            {type="fluid", name=f.name, amount=barrel_size},
            {type="item", name=barrel_type, amount=1}
          }
        }
      })

    -- add recipe unlocks to fluid handling technology
    table.insert
    (
      data.raw["technology"][techname].effects,
      {type = "unlock-recipe", recipe = fill_name}
    )
    table.insert
    (
      data.raw["technology"][techname].effects,
      {type = "unlock-recipe", recipe = empty_name}
    )

  end --existing_canister
end
